package com.retmix.volga.shared.controllers;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.dto.admin.AccountDTO;
import com.retmix.volga.shared.dto.admin.UpsertAccountDTO;
import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.shared.services.AdminService;
import com.retmix.volga.shared.services.UserService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.annotation.Nullable;
import lombok.AllArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.security.NoSuchAlgorithmException;
import java.util.List;

@RestController
@RequestMapping("/Admin/Account")
@AllArgsConstructor
public class AdminAccountController {
    private final AdminService adminService;
    private UserService userService;

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping
    public List<AccountDTO> index(@RequestHeader("authorization") String tokenRequest, @RequestParam("email") @Nullable String email, @RequestParam("start") int start, @RequestParam("start") int count) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return adminService.getAll(email, start, count);
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/{userId}")
    public AccountDTO index(@RequestHeader("authorization") String tokenRequest, @PathVariable Long userId) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return adminService.byId(userId);
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PostMapping
    public ResponseEntity<AccountDTO> create(@RequestHeader("authorization") String tokenRequest, @RequestBody UpsertAccountDTO data) throws NoSuchAlgorithmException {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.status(201).body(adminService.create(data));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PutMapping("/{userId}")
    public ResponseEntity<AccountDTO> update(@RequestHeader("authorization") String tokenRequest, @RequestBody UpsertAccountDTO data, @PathVariable Long userId) throws NoSuchAlgorithmException {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(adminService.update(data, userId));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @DeleteMapping("/{userId}")
    public ResponseEntity<MessageDTO> delete(@RequestHeader("authorization") String tokenRequest, @PathVariable Long userId) throws NoSuchAlgorithmException {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(adminService.delete(userId));
    }

}
