package com.retmix.volga.shared.models;

import com.retmix.volga.shared.dto.EnterDTO;
import com.retmix.volga.shared.dto.UserDTO;
import com.retmix.volga.shared.dto.admin.AccountDTO;
import com.retmix.volga.storage.models.Directory;
import com.retmix.volga.storage.models.File;
import jakarta.persistence.*;
import lombok.*;

import java.util.List;

@Entity
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@Table(name = "users")
public class User {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    private String email;
    private String password;
    private String token;
    @Column(name = "is_admin")
    private boolean isAdmin;
    @Column(name = "disk_size")
    private Long diskSize;
    @OneToMany(mappedBy = "user", fetch = FetchType.EAGER)
    private List<Directory> dirsUser;
    @OneToMany(mappedBy = "user", fetch = FetchType.EAGER)
    private List<File> userFiles;

    public UserDTO toDTO() {
        return UserDTO.builder()
                .id(id)
                .username(email)
                .isAdmin(isAdmin).build();
    }

    public AccountDTO toAdminDTO() {
        return AccountDTO.builder()
                .email(email)
                .admin(isAdmin)
                .diskSize(diskSize).build();
    }
    public EnterDTO toEnterDTO() {
        return EnterDTO.builder().id(id).email(email).token(token).build();
    }
}
