package com.retmix.volga.shared.services;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.dto.admin.AccountDTO;
import com.retmix.volga.shared.dto.admin.UpsertAccountDTO;
import com.retmix.volga.shared.handler.exceptions.ObjectNotFoundException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.shared.repositories.UserRepository;
import com.retmix.volga.utils.PasswordHash;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.security.NoSuchAlgorithmException;
import java.util.List;

@Service
@AllArgsConstructor
public class AdminService {
    private final UserRepository userRepository;
    private final PasswordHash passwordHash;
    public List<AccountDTO> getAll(String email, int start, int count) {
        if (email == null) {
            return userRepository.findAll()
                    .stream().map(User::toAdminDTO)
                    .toList().stream().limit(count).toList();
        }

        var data = userRepository.findUserByEmail(email);
        if (data.isEmpty()) {
            return userRepository.findAll().stream().map(User::toAdminDTO)
                    .toList().stream().limit(count).toList();
        }
        else {
            return userRepository.findAll().stream().filter(item -> item.getEmail().equals(email))
                    .toList().stream().map(User::toAdminDTO).toList();
        }
    }

    public AccountDTO byId(Long id) {
        return userRepository.findById(id).orElseThrow(() -> new ObjectNotFoundException("Not found")).toAdminDTO();
    }

    public AccountDTO create(UpsertAccountDTO data) throws NoSuchAlgorithmException {
        User user = User.builder()
                .email(data.getEmail())
                .password(passwordHash.hashPassword(data.getPassword()))
                .isAdmin(data.getIsAdmin())
                .diskSize(data.getDiskSize()).build();

        return userRepository.save(user).toAdminDTO();
    }

    public AccountDTO update(UpsertAccountDTO data, Long id) throws NoSuchAlgorithmException {
        User foundUser = userRepository.findById(id).orElseThrow(()-> new ObjectNotFoundException("Not found"));

        foundUser.setEmail(data.getEmail());
        foundUser.setPassword(passwordHash.hashPassword(data.getPassword()));
        foundUser.setAdmin(data.getIsAdmin());
        foundUser.setDiskSize(data.getDiskSize());
        return userRepository.save(foundUser).toAdminDTO();
    }

    public MessageDTO delete(Long id) {
        User foundUser = userRepository.findById(id).orElseThrow(()-> new ObjectNotFoundException("Not found"));
        userRepository.delete(foundUser);
        return new MessageDTO("User deleted");
    }




}
