package com.retmix.volga.storage.controllers;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.services.UserService;
import com.retmix.volga.storage.dto.DirDTO;
import com.retmix.volga.storage.dto.UpsertDirDTO;
import com.retmix.volga.storage.services.DirectoryService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.validation.Valid;
import lombok.AllArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/Directory")
@AllArgsConstructor
public class DirectoryController {
    private final UserService userService;
    private final DirectoryService directoryService;

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/GetRoot")
    public ResponseEntity<DirDTO> getRoot(@RequestHeader("authorization") String tokenRequest) {
        return ResponseEntity.ok(directoryService.getRoot(userService.getUserByToken(tokenRequest)));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/{id}")
    public ResponseEntity<DirDTO> getById(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id) {
        return ResponseEntity.ok(directoryService.getById(userService.getUserByToken(tokenRequest), id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/GetByDirectory/{dirId}")
    public ResponseEntity<List<DirDTO>> getSubDirs(@RequestHeader("authorization") String tokenRequest, @PathVariable Long dirId) {
        return ResponseEntity.ok(directoryService.getSubsidiaryDirs(userService.getUserByToken(tokenRequest), dirId));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PostMapping
    public ResponseEntity<DirDTO> create(@RequestHeader("authorization") String tokenRequest, @RequestBody @Valid UpsertDirDTO data) {
        return ResponseEntity.status(201).body(directoryService.create(userService.getUserByToken(tokenRequest), data));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PatchMapping("/{id}")
    public ResponseEntity<DirDTO> update(@RequestHeader("authorization") String tokenRequest, @RequestBody @Valid UpsertDirDTO data, @PathVariable Long id) {
        return ResponseEntity.ok(directoryService.update(userService.getUserByToken(tokenRequest), data, id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @DeleteMapping("/{id}")
    public ResponseEntity<MessageDTO> delete(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id) {
        return ResponseEntity.ok(directoryService.delete(userService.getUserByToken(tokenRequest), id));
    }
}
