package com.retmix.volga.storage.controllers;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.services.UserService;
import com.retmix.volga.storage.dto.files.FileDTO;
import com.retmix.volga.storage.dto.files.GetFileDTO;
import com.retmix.volga.storage.dto.files.UpsertFileDTO;
import com.retmix.volga.storage.services.FileService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.annotation.Nullable;
import jakarta.validation.constraints.Null;
import lombok.AllArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/Files")
@AllArgsConstructor
public class FileController {
    private final FileService fileService;
    private final UserService userService;

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/{id}")
    public ResponseEntity<FileDTO> getById(@RequestHeader("authorization") String tokenRequest,
                                           @RequestBody @Nullable GetFileDTO data, @PathVariable Long id) {
        return ResponseEntity.ok(fileService.getFileId(userService.getUserByToken(tokenRequest), data, id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/GetByDirectory/{dirId}")
    public ResponseEntity<List<FileDTO>> getFromDir(@RequestHeader("authorization") String tokenRequest,
                                                    @PathVariable Long dirId) {
        return ResponseEntity.ok(fileService.getFromDir(userService.getUserByToken(tokenRequest), dirId));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PostMapping
    public ResponseEntity<FileDTO> create(@RequestHeader("authorization") String tokenRequest, @RequestBody UpsertFileDTO data) {
        return ResponseEntity.status(201).body(fileService.create(userService.getUserByToken(tokenRequest), data));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PutMapping("/{id}")
    public ResponseEntity<FileDTO> update(@RequestHeader("authorization") String tokenRequest, @RequestBody UpsertFileDTO data, @PathVariable Long id) {
        return ResponseEntity.ok(fileService.update(userService.getUserByToken(tokenRequest), data, id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @DeleteMapping("/{id}")
    public ResponseEntity<MessageDTO> delete(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id) {
        return ResponseEntity.ok(fileService.delete(userService.getUserByToken(tokenRequest), id));
    }
}
