package com.retmix.volga.storage.controllers;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.services.UserService;
import com.retmix.volga.storage.dto.StorageDTO;
import com.retmix.volga.storage.dto.files.FileDTO;
import com.retmix.volga.storage.services.StorageService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import lombok.AllArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@RestController
@RequestMapping("/Storage")
@AllArgsConstructor
public class StorageController {
    private final UserService userService;
    private final StorageService storageService;

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/Info")
    public ResponseEntity<StorageDTO> infoStorage(@RequestHeader("authorization") String tokenRequest) {
        return ResponseEntity.ok(storageService.info(userService.getUserByToken(tokenRequest)));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/Download/{id}")
    public ResponseEntity<Resource> download(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id, @RequestParam String password) throws IOException {
        var data = storageService.downloadFile(userService.getUserByToken(tokenRequest), password, id);
        String contentType = "application/octet-stream";
        String headerValue = "attachment; filename=\"" + data.getFilename() + "\"";

        return ResponseEntity.ok()
                .contentType(MediaType.parseMediaType(contentType))
                .header(HttpHeaders.CONTENT_DISPOSITION, headerValue)
                .body(data);
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PutMapping(path = "/Upload/{id}", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ResponseEntity<FileDTO> uploadFile(@RequestHeader("authorization") String tokenRequest, @RequestPart("file") MultipartFile file, @PathVariable Long id) {
        return ResponseEntity.ok(storageService.upload(userService.getUserByToken(tokenRequest), id, file));
    }
}
