package com.retmix.volga.storage.controllers.admin;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.shared.services.UserService;
import com.retmix.volga.storage.dto.DirDTO;
import com.retmix.volga.storage.dto.UpsertDirDTO;
import com.retmix.volga.storage.services.AdminDirService;
import com.retmix.volga.storage.services.DirectoryService;
import com.retmix.volga.storage.services.FileService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.validation.Valid;
import lombok.AllArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/Admin/Directory")
@AllArgsConstructor
public class AdminDirectoryController {
    private final UserService userService;
    private final DirectoryService directoryService;
    private final AdminDirService adminDirService;


    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/GetRoot/{userId}")
    public ResponseEntity<DirDTO> getRoot(@RequestHeader("authorization") String tokenRequest, @PathVariable Long userId) {
        return ResponseEntity.ok(adminDirService.getRoot(userId));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/{id}")
    public ResponseEntity<DirDTO> getByIdDir(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(adminDirService.getById(id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/GetByDirectory/{dirId}")
    public ResponseEntity<List<DirDTO>> getSubDirs(@RequestHeader("authorization") String tokenRequest, @PathVariable Long dirId) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(directoryService.getSubsidiaryDirs(user, dirId));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PostMapping
    public ResponseEntity<DirDTO> create(@RequestHeader("authorization") String tokenRequest, @RequestBody @Valid UpsertDirDTO data) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }
        return ResponseEntity.status(201).body(directoryService.create(user, data));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PatchMapping("/{id}")
    public ResponseEntity<DirDTO> update(@RequestHeader("authorization") String tokenRequest, @RequestBody @Valid UpsertDirDTO data, @PathVariable Long id) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }
        return ResponseEntity.ok(directoryService.update(user, data, id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @DeleteMapping("/{id}")
    public ResponseEntity<MessageDTO> delete(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }
        return ResponseEntity.ok(directoryService.delete(user, id));
    }
}
