package com.retmix.volga.storage.controllers.admin;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.shared.services.UserService;
import com.retmix.volga.storage.dto.files.FileDTO;
import com.retmix.volga.storage.dto.files.GetFileDTO;
import com.retmix.volga.storage.dto.files.UpsertFileDTO;
import com.retmix.volga.storage.services.AdminFileService;
import com.retmix.volga.storage.services.FileService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import lombok.AllArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/Admin/Files")
@AllArgsConstructor
public class AdminFileController {
    private final FileService fileService;
    private final UserService userService;
    private final AdminFileService adminFileService;

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/{id}")
    public ResponseEntity<FileDTO> getById(@RequestHeader("authorization") String tokenRequest,
                                           @RequestBody GetFileDTO data, @PathVariable Long id) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }
        return ResponseEntity.ok(adminFileService.getFileId(id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/GetByDirectory/{dirId}")
    public ResponseEntity<List<FileDTO>> getFromDir(@RequestHeader("authorization") String tokenRequest,
                                                    @PathVariable Long dirId) {

        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(adminFileService.getFromDir(dirId));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PostMapping
    public ResponseEntity<FileDTO> create(@RequestHeader("authorization") String tokenRequest, @RequestBody UpsertFileDTO data) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }
        return ResponseEntity.status(201).body(fileService.create(user, data));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PutMapping("/{id}")
    public ResponseEntity<FileDTO> update(@RequestHeader("authorization") String tokenRequest, @RequestBody UpsertFileDTO data, @PathVariable Long id) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(fileService.update(user, data, id));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @DeleteMapping("/{id}")
    public ResponseEntity<MessageDTO> delete(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(fileService.delete(user, id));
    }
}
