package com.retmix.volga.storage.controllers.admin;

import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.shared.services.UserService;
import com.retmix.volga.storage.dto.StorageDTO;
import com.retmix.volga.storage.dto.files.FileDTO;
import com.retmix.volga.storage.services.AdminStorageService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import lombok.AllArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@RestController
@RequestMapping("/Admin/Storage")
@AllArgsConstructor
public class AdminStorageController {
    private UserService userService;
    private AdminStorageService adminStorageService;

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/Info/{userId}")
    public ResponseEntity<StorageDTO> infoStorage(@RequestHeader("authorization") String tokenRequest, @PathVariable Long userId) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(adminStorageService.info(userId));
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @GetMapping("/Download/{id}")
    public ResponseEntity<Resource> download(@RequestHeader("authorization") String tokenRequest, @PathVariable Long id, @RequestParam String password) throws IOException {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        var data = adminStorageService.downloadFile(id);
        String contentType = "application/octet-stream";
        String headerValue = "attachment; filename=\"" + data.getFilename() + "\"";

        return ResponseEntity.ok()
                .contentType(MediaType.parseMediaType(contentType))
                .header(HttpHeaders.CONTENT_DISPOSITION, headerValue)
                .body(data);
    }

    @SecurityRequirement(
            name = "bearerAuth"
    )
    @PutMapping(path = "/Upload/{id}", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ResponseEntity<FileDTO> uploadFile(@RequestHeader("authorization") String tokenRequest, @RequestPart("file") MultipartFile file, @PathVariable Long id) {
        User user = userService.getUserByToken(tokenRequest);
        if (!user.isAdmin()) {
            throw new PermissionException("Access denied");
        }

        return ResponseEntity.ok(adminStorageService.upload(id, file));
    }
}
