package com.retmix.volga.storage.services;

import com.retmix.volga.shared.handler.exceptions.ObjectNotFoundException;
import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.handler.exceptions.ValidationException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.shared.repositories.UserRepository;
import com.retmix.volga.storage.dto.StorageDTO;
import com.retmix.volga.storage.dto.files.FileDTO;
import com.retmix.volga.storage.models.File;
import com.retmix.volga.storage.repositories.FileRepository;
import com.retmix.volga.utils.FileUtils;
import lombok.AllArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@Service
@AllArgsConstructor
public class AdminStorageService {
    private final FileRepository fileRepository;
    private final FileUtils fileUtils;
    private final UserRepository userRepository;

    public StorageDTO info(Long userId) {
        User user = userRepository.findById(userId)
                .orElseThrow(() -> new ObjectNotFoundException("Not found"));

        return StorageDTO.builder()
                .total(1073741824L)
                .busy(user.getDiskSize()).build();
    }

    public FileDTO upload( Long fileId, MultipartFile fileUpload) {
        File file = fileRepository.findById(fileId).orElseThrow(()-> new ObjectNotFoundException("Not found"));

        if (file.getUser().getDiskSize()< fileUpload.getSize()) {
            throw new ValidationException("Not enough disk space");
        }

        String urlFile = fileUtils.saveFile(fileUpload);
        User user = file.getUser();
        file.setUrl(urlFile);
        user.setDiskSize(user.getDiskSize() - fileUpload.getSize());
        userRepository.save(user);
        return fileRepository.save(file).toDTO();

    }

    public Resource downloadFile(Long fileId) throws IOException {
        File file = fileRepository.findById(fileId)
                .orElseThrow(() -> new ObjectNotFoundException("Not found"));
        return fileUtils.getFileByName(file.getUrl());

    }
}
