package com.retmix.volga.storage.services;

import com.retmix.volga.shared.handler.exceptions.ObjectNotFoundException;
import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.handler.exceptions.ValidationException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.storage.dto.StorageDTO;
import com.retmix.volga.storage.dto.files.FileDTO;
import com.retmix.volga.storage.models.File;
import com.retmix.volga.storage.repositories.FileRepository;
import com.retmix.volga.utils.FileUtils;
import lombok.AllArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@Service
@AllArgsConstructor
public class StorageService {
    private final FileRepository fileRepository;
    private final FileUtils fileUtils;

    public StorageDTO info(User user) {
        return StorageDTO.builder()
                .total(1073741824L)
                .busy(user.getDiskSize()).build();
    }

    public FileDTO upload(User user, Long fileId, MultipartFile fileUpload) {
        File file = user.getUserFiles().stream().filter(item -> item.getId().equals(fileId)).findFirst()
                .orElseThrow(() -> new ObjectNotFoundException("Not found"));
        if (user.getDiskSize()< fileUpload.getSize()) {
            throw new ValidationException("Not enough disk space");
        }

        String urlFile = fileUtils.saveFile(fileUpload);
        file.setUrl(urlFile);
        user.setDiskSize(user.getDiskSize() - fileUpload.getSize());
        return fileRepository.save(file).toDTO();

    }

    public Resource downloadFile(User user, String password, Long fileId) throws IOException {

        if (user.getUserFiles().stream().filter(item -> item.getId().equals(fileId)).findFirst().isEmpty()){
            File file = fileRepository.findById(fileId).orElseThrow(() -> new ObjectNotFoundException("Not found"));
            if (!file.isShared()) {
                throw new PermissionException("Access denied");
            }

            if (!file.getPassword().equals(password)){
                throw new PermissionException("Access denied");
            }

            return fileUtils.getFileByName(file.getUrl());
        }

        File file = user.getUserFiles().stream().filter(item -> item.getId().equals(fileId)).findFirst()
                .orElseThrow(() -> new ObjectNotFoundException("Not found"));

        return fileUtils.getFileByName(file.getUrl());

    }
}
