package com.retmix.volga.storage.services.impl;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.handler.exceptions.ObjectNotFoundException;
import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.storage.dto.DirDTO;
import com.retmix.volga.storage.dto.UpsertDirDTO;
import com.retmix.volga.storage.models.Directory;
import com.retmix.volga.storage.repositories.DirectoryRepository;
import com.retmix.volga.storage.services.DirectoryService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class DirServiceImpl implements DirectoryService {
    private final DirectoryRepository directoryRepository;
    @Override
    public DirDTO getRoot(User user) {

        return user.getDirsUser().stream().filter(Directory::isRoot).map(Directory::toDTO).toList().stream().findFirst()
                .get();
    }

    @Override
    public DirDTO getById(User user, Long id) {
        return user.getDirsUser().stream().filter(item -> item.getId().equals(id)).findFirst()
                .orElseThrow(() -> new ObjectNotFoundException("Not found")).toDTO();
    }

    @Override
    public List<DirDTO> getSubsidiaryDirs(User user, Long parentId) {
        return user.getDirsUser().stream().filter(item -> item.getId().equals(parentId)).findFirst()
                .orElseThrow(() -> new ObjectNotFoundException("Not found")).getInnerDirs().stream().map(Directory::toDTO)
                .toList();
    }

    @Override
    public DirDTO create(User user, UpsertDirDTO data) {
        Directory parDir = directoryRepository.findById(data.getParentId()).orElseThrow(() -> new ObjectNotFoundException("Not found"));
        if (!parDir.getUser().equals(user)) {
            throw new PermissionException("Access den");
        }
        Directory directory = Directory.builder()
                .name(data.getName())
                .root(false)
                .user(user)
                .directory(parDir).build();

        return directoryRepository.save(directory).toDTO();
    }

    @Override
    public DirDTO update(User user, UpsertDirDTO data, Long dirId) {
        Directory parDir = directoryRepository.findById(data.getParentId()).orElseThrow(() -> new ObjectNotFoundException("Not found"));
        if (!parDir.getUser().equals(user)) {
            throw new PermissionException("Access den");
        }
        Directory updateDir = directoryRepository.findById(dirId).orElseThrow(() -> new ObjectNotFoundException("Not found"));
        if (!updateDir.getUser().equals(user)) {
            throw new PermissionException("Access den");
        }

        updateDir.setName(data.getName());
        updateDir.setDirectory(parDir);
        return directoryRepository.save(updateDir).toDTO();
    }

    @Override
    public MessageDTO delete(User user, Long dirId) {
        Directory delDir = directoryRepository.findById(dirId).orElseThrow(() -> new ObjectNotFoundException("Not found"));
        if (!delDir.getUser().equals(user)) {
            throw new ObjectNotFoundException("Not found");
        }
        directoryRepository.delete(delDir);

        return new MessageDTO("Dir deleted");
    }
}
