package com.retmix.volga.storage.services.impl;

import com.retmix.volga.shared.dto.MessageDTO;
import com.retmix.volga.shared.handler.exceptions.ObjectNotFoundException;
import com.retmix.volga.shared.handler.exceptions.PermissionException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.storage.dto.files.FileDTO;
import com.retmix.volga.storage.dto.files.GetFileDTO;
import com.retmix.volga.storage.dto.files.UpsertFileDTO;
import com.retmix.volga.storage.models.Directory;
import com.retmix.volga.storage.models.File;
import com.retmix.volga.storage.repositories.DirectoryRepository;
import com.retmix.volga.storage.repositories.FileRepository;
import com.retmix.volga.storage.services.FileService;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@AllArgsConstructor
public class FileServiceImpl implements FileService {
    private final DirectoryRepository directoryRepository;
    private final FileRepository fileRepository;
    @Override
    public FileDTO getFileId(User user, GetFileDTO data, Long id) {

        if (user.getUserFiles().stream().filter(item -> item.getId().equals(id)).findFirst().isEmpty()){
            File file = fileRepository.findById(id).orElseThrow(() -> new ObjectNotFoundException("Not found"));
            if (!file.isShared()) {
                throw new PermissionException("Access denied");
            }

            if (!file.getPassword().equals(data.getPassword())){
                throw new PermissionException("Access denied");
            }

            return file.toDTO();
        }

        return user.getUserFiles().stream()
                .filter(item -> item.getId().equals(id)).findFirst()
                .orElseThrow(() -> new ObjectNotFoundException("Not found"))
                .toDTO();
    }

    @Override
    public List<FileDTO> getFromDir(User user, Long idDir) {
        return user.getDirsUser().stream().filter(item -> item.getId().equals(idDir))
                .findFirst().orElseThrow(() -> new ObjectNotFoundException("Not found"))
                .getFilesInDir().stream().map(File::toDTO).toList();
    }

    @Override
    public FileDTO create(User user, UpsertFileDTO data) {
        Directory directory = user.getDirsUser().stream().filter(item -> item.getId().equals(data.getDirectoryId()))
                .findFirst().orElseThrow(() -> new ObjectNotFoundException("Not found"));
        File file = File.builder()
                .name(data.getName())
                .url(RandomStringUtils.randomAlphabetic(10))
                .password(data.getPassword())
                .shared(data.getShared())
                .directory(directory)
                .user(user)
                .build();
        return fileRepository.save(file).toDTO();
    }

    @Override
    public FileDTO update(User user, UpsertFileDTO data, Long id) {
        Directory directory = user.getDirsUser().stream().filter(item -> item.getId().equals(data.getDirectoryId()))
                .findFirst().orElseThrow(() -> new ObjectNotFoundException("Not found"));
        File foundFile = fileRepository.findById(id)
                .orElseThrow(() -> new ObjectNotFoundException("Not found"));
        if (!foundFile.getUser().equals(user)) {
            throw new ObjectNotFoundException("Not found");
        }

        foundFile.setName(data.getName());
        foundFile.setPassword(data.getPassword());
        foundFile.setShared(data.getShared());
        foundFile.setDirectory(directory);
        return fileRepository.save(foundFile).toDTO();
    }

    @Override
    public MessageDTO delete(User user, Long id) {
        File file = user.getUserFiles().stream()
                .filter(item -> item.getId().equals(id))
                .findFirst().orElseThrow(() -> new ObjectNotFoundException("Not found"));
        fileRepository.delete(file);
        return new MessageDTO("File deleted");
    }
}
