package com.retmix.volga.utils;

import com.retmix.volga.shared.handler.exceptions.ObjectNotFoundException;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.concurrent.atomic.AtomicReference;

@Component

public class FileUtils {
    private final String fileDir = "storage";
    private final Path path;

    public FileUtils() throws IOException, URISyntaxException {
        path = Paths.get(fileDir);
        if (!Files.exists(path)) {
            Files.createDirectory(path);
        }
    }

    public String saveFile(MultipartFile file) {
        String fileCode = RandomStringUtils.randomAlphabetic(10);
        try (InputStream inputStream = file.getInputStream()) {
            String originalFileName = file.getOriginalFilename();
            Path filePath = path.resolve(fileCode + originalFileName.substring(originalFileName.indexOf('.')));
            Files.copy(inputStream, filePath, StandardCopyOption.REPLACE_EXISTING);
            return filePath.toString();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public Resource getFileByName(String fileName) throws IOException {
        var pathFile = Files.list(path).toList()
                .stream()
                .filter(f -> f.getFileName().toString().startsWith(fileName)).findFirst();

        return new UrlResource(pathFile.orElseThrow(()-> new ObjectNotFoundException("File not found")).toUri());
    }
}
